%
%  This script plots the evolution of the node counts (or percentages)
%    of the total nodes in the tree, # of leaves, and # of cleared nodes
%
%  Used to generate the following figures for 2011-ICRA paper
%   - 'fig-NodesTotalVsLeaves_MaxNodes300.png'
%   - 'fig-NodesTotalVsCleared_MaxNodes300.png'
%   - 'fig-NodesTotalVsPruned_MaxNodes300.png'
%
close all; clc; clc;

FLAG_FIG1 = true;
FLAG_FIG2 = true;
FLAG_FIG3 = true;

% Load the data file for the hierarchical search simulation runs
datafile = 'HierarchicalSearch_MaxNodes300';    %
% datafile = 'Set2';    %
% datafile = 'Set3';    %
load( datafile );


% Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
%  tree_size
%  number_of_leaves
%  number_of_cleared
correct_mask_x = (final_positions(1,:) == 200.5);
correct_mask_y = (final_positions(2,:) == 200.5);
correct_mask = correct_mask_x & correct_mask_y;

% Fix all data with only correct searches
time_steps(~correct_mask) = [];
tree_size(~correct_mask,:) = [];
number_of_leaves(~correct_mask,:) = [];
number_of_cleared(~correct_mask,:) = [];




if(FLAG_FIG1)
    %--------------
    % Compute statistics on expected time till decision
    [N,X] = hist( time_steps, 1:20:1000 );
    
    %==============
    % Plot the time till decision distribution
    figure;
    set(gcf, 'color','white');
    
    h_bar = bar( X, N./sum(N), 0.8 );
    
    set(gca, 'XLim', [1, size(tree_size,2)], 'YLim', [0 1.10*max(N./sum(N))]);
    set(gca, 'FontSize', 14, 'LineWidth', 3)
    
    % - plot mean, std, median stats
    h_text(1) = text( 500, 0.070, 'Max Initial Nodes = 300');
    h_text(2) = text( 600, 0.065, sprintf('Mean: %3.2f', mean(time_steps) ) );
    h_text(3) = text( 600, 0.060, sprintf('STD: %3.2f', std(time_steps,1) ) );
    h_text(4) = text( 600, 0.055, sprintf('Median: %d', median(time_steps) ) );
    set(h_text, 'FontSize', 14);
    set(h_text(1), 'FontWeight', 'bold');
    
    xlabel('Time step, t  '); ylabel('Pr[TTD = t], Decision Time Distribution  ');
end



%==============
% Need to adaptively determine statistics, since some replications
% terminate before others
%
num_runs = zeros( 1, size(tree_size,2) );       % keeps track of samples in statistic
tree_size_avg = zeros( 1, size(tree_size,2) );
tree_size_std = zeros( 1, size(tree_size,2) );
number_of_leaves_avg = zeros( 1, size(tree_size,2) );
number_of_leaves_std = zeros( 1, size(tree_size,2) );
number_of_cleared_avg = zeros( 1, size(tree_size,2) );
number_of_cleared_std = zeros( 1, size(tree_size,2) );

for t = 1:size(tree_size,2)
    % Determine valid samples
    valid_run_ind = find( tree_size(:,t) );
    
    % Determine number of valid samples at this time step
    %  - if run has already terminated, should contain zero
    num_runs(t) = length( valid_run_ind );
    
    % Compute statistics for the valid samples at this time step
    
    
    %--------------
    % Compute statistics on tree size at every time step
    tree_size_avg(t) = mean(tree_size( valid_run_ind, t ), 1 );
    tree_size_std(t) = std(tree_size( valid_run_ind, t ), 1, 1);
    
    
    %--------------
    % Compute statistics on number of leaves at every time step
    number_of_leaves_avg(t) = mean( number_of_leaves( valid_run_ind, t ), 1 );
    number_of_leaves_std(t) = std( number_of_leaves( valid_run_ind, t ), 1, 1 );
    
    
    %--------------
    % Compute statistics on number of cleared nodes at every time step
    number_of_cleared_avg(t) = mean( number_of_cleared( valid_run_ind, t ), 1 );
    number_of_cleared_std(t) = std( number_of_cleared( valid_run_ind, t ), 1, 1 );
    
    
    
end         % end for(t)

% Clean up any erroneous statistics (i.e., no samples)
tree_size_avg(isnan(tree_size_avg)) = [];
tree_size_std(isnan(tree_size_std)) = [];
number_of_leaves_avg(isnan(number_of_leaves_avg)) = [];
number_of_leaves_std(isnan(number_of_leaves_std)) = [];
number_of_cleared_avg(isnan(number_of_cleared_avg)) = [];
number_of_cleared_std(isnan(number_of_cleared_std)) = [];

if(FLAG_FIG2)
    
    %==============
    % Plot the average tree size
    figure;
    set(gcf, 'color','white', 'Position', [360 380 582 298]);
    hold on;
    
    % Plot the average tree size evolution
    h_plot1 = plot( 1:size(tree_size_avg,2), tree_size_avg, 'b' );
    h_plot1u = plot( 1:size(tree_size_avg,2), tree_size_avg+3*tree_size_std, 'b-' );
    h_plot1l = plot( 1:size(tree_size_avg,2), tree_size_avg-3*tree_size_std, 'b-' );
    
    set(h_plot1, 'LineWidth', 3);
    set(h_plot1u, 'LineWidth', 1); set(h_plot1l, 'LineWidth', 1);
    
    
    % Plot the average number of leaves evolution
    h_plot2 = plot( 1:size(number_of_leaves_avg,2), number_of_leaves_avg, 'r' );
    h_plot2u = plot( 1:size(number_of_leaves_avg,2), number_of_leaves_avg+3*number_of_leaves_std, 'r-' );
    h_plot2l = plot( 1:size(number_of_leaves_avg,2), number_of_leaves_avg-3*number_of_leaves_std, 'r-' );
    
    set(h_plot2, 'LineWidth', 3);
    set(h_plot2u, 'LineWidth', 1); set(h_plot2l, 'LineWidth', 1);
    
    
    % Define legend
    % h_legend = legend( [h_plot1, h_plot2, h_plot1u, h_plot2u], 'Total # of nodes  ', 'Total # of leaves  ', '3\sigma bounds on # total nodes', '3\sigma bounds on # leaf nodes', 'Location', 'NorthWest' );
    h_legend = legend( [h_plot1, h_plot2], 'Total # of nodes   ', 'Total # of leaves  ', 'Location', 'NorthWest' );
    set(h_legend, 'FontSize', 14 )
    
    
    
    
    
    % % Construct the variability patch for tree_size_avg
    % X = [1:size(tree_size_avg,2), size(tree_size_avg,2):-1:1]';
    % Y = [tree_size_avg, zeros(size(tree_size_avg))]';
    % h_patch = patch(X,Y,'b');
    % set(h_patch, 'FaceAlpha', 0.2);
    %
    %
    % % Construct the variability patch for number_of_leaves_avg
    % X = [1:size(number_of_leaves,2), size(number_of_leaves,2):-1:1]';
    % Y = [number_of_leaves_avg, zeros(size(number_of_leaves_avg))]';
    % h_patch2 = patch(X,Y,'r');
    % set(h_patch2, 'FaceAlpha', 0.2);
    
    
    
    minY = min( min(tree_size_avg), 0 );
    maxY = max( max(tree_size_avg), max(number_of_leaves_avg) );
    set(gca, 'XLim', [1, size(tree_size,2)], 'YLim', [0.95*minY 1.10*maxY]);
    set(gca, 'FontSize', 14, 'LineWidth', 3)
    
    xlabel('Time step, t  '); ylabel('Number of nodes  ');
    
end


if(FLAG_FIG3)
    
    %==============
    % Plot the average tree size
    figure;
    set(gcf, 'color','white', 'Position', [360 380 582 298]);
    
    hold on;
    % Plot the average tree size evolution
    h_plot1 = plot( 1:size(tree_size_avg,2), tree_size_avg, 'b' );
    h_plot1u = plot( 1:size(tree_size_avg,2), tree_size_avg+3*tree_size_std, 'b-' );
    h_plot1l = plot( 1:size(tree_size_avg,2), tree_size_avg-3*tree_size_std, 'b-' );
    
    set(h_plot1, 'LineWidth', 3);
    set(h_plot1u, 'LineWidth', 1); set(h_plot1l, 'LineWidth', 1);
    
    
    % Plot the average number of cleared evolution
    h_plot2 = plot( 1:size(number_of_cleared_avg,2), number_of_cleared_avg, 'r' );
    h_plot2u = plot( 1:size(number_of_cleared_avg,2), number_of_cleared_avg+3*number_of_cleared_std, 'r-' );
    h_plot2l = plot( 1:size(number_of_cleared_avg,2), number_of_cleared_avg-3*number_of_cleared_std, 'r-' );
    
    set(h_plot2, 'LineWidth', 3);
    set(h_plot2u, 'LineWidth', 1); set(h_plot2l, 'LineWidth', 1);
    
    
    
    % Construct the variability patch for tree_size_avg
    X = [1:size(tree_size_avg,2), size(tree_size_avg,2):-1:1, ]';
    Y = [tree_size_avg, fliplr(number_of_cleared_avg)]';
    h_patch = patch(X,Y,'b');
    set(h_patch, 'FaceAlpha', 0.2, 'FaceColor', 'b', 'EdgeColor', 'b');
    
    % % Construct the variability patch for number_of_cleared_avg
    % X = [1:size(number_of_cleared,2), size(number_of_cleared,2):-1:1]';
    % Y = [number_of_cleared_avg, zeros(size(number_of_cleared_avg))]';
    % h_patch2 = patch(X,Y,'r');
    % set(h_patch2, 'FaceAlpha', 0.2);
    
    
    % Define legend
    h_legend = legend( [h_plot1, h_plot2, h_patch], ...
        'Total # of nodes  :', 'Total # of cleared nodes  :', 'Total # of active nodes  :', ...
        'Location', 'SouthEast' );
    set(h_legend, 'FontSize', 14 )
    
    
    
    minY = min( min(tree_size_avg), min(number_of_cleared_avg) );
    maxY = max( max(tree_size_avg), max(number_of_cleared_avg) );
    set(gca, 'XLim', [1, size(tree_size,2)], 'YLim', [0.95*minY 1.10*maxY]);
    set(gca, 'FontSize', 14, 'LineWidth', 3)
    
    xlabel('Time step, t  '); ylabel('Number of nodes  ');
    
end