%
%  This script plots the distribution of decision times for the uniform grid
%       against that of the hierarchical approach
%
%  Used to generate the following figures for 2011-ICRA paper
%   - 'fig-BeliefEvol_MaxNodes300.png'
%
close all; clc; clc;

% Load the data file for the uniform search simulation runs
datafile = 'allUniformDepth7.mat';    %
load( datafile );




% Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
%  tree_size
%  number_of_leaves
%  number_of_cleared
correct_mask_x = (final_positions(1,:) == 202);
correct_mask_y = (final_positions(2,:) == 202);
correct_mask = correct_mask_x & correct_mask_y;

% Fix all data with only correct searches
time_steps(~correct_mask) = [];
tree_size(~correct_mask,:) = [];
number_of_leaves(~correct_mask,:) = [];
number_of_cleared(~correct_mask,:) = [];


%--------------
% Compute statistics on expected time till decision
[N,X] = hist( time_steps, 1:20:1000 );

%==============
% Plot the time till decision distribution
figure;
set(gcf, 'color','white', 'Position', [360 389 667 289]);

h_bar = bar( X, N./sum(N), 0.8 );
hold on;
set(gca, 'XLim', [1, size(tree_size,2)], 'YLim', [0 1.10*max(N./sum(N))]);
set(gca, 'FontSize', 14, 'LineWidth', 3)

% % - plot mean, std, median stats
% h_text(1) = text( 500, 0.070, 'Max Initial Nodes = 300');
% h_text(2) = text( 600, 0.065, sprintf('Mean: %3.2f', mean(time_steps) ) );
% h_text(3) = text( 600, 0.060, sprintf('STD: %3.2f', std(time_steps,1) ) );
% h_text(4) = text( 600, 0.055, sprintf('Median: %d', median(time_steps) ) );
% set(h_text, 'FontSize', 14);
% set(h_text(1), 'FontWeight', 'bold');

xlabel('Time step, t  '); ylabel('Pr[TTD = t] ');
stats7 = [mean(time_steps), std(time_steps,1), median(time_steps)];
fprintf('Mean: %3.2f, STD: %3.2f, Median: %d\n', mean(time_steps), std(time_steps,1), median(time_steps) );







% Load the data file for the hierarchical search simulation runs
datafile = 'allUniformDepth6.mat';    %
load( datafile );

% Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
%  tree_size
%  number_of_leaves
%  number_of_cleared
correct_mask_x = (final_positions(1,:) == 204);
correct_mask_y = (final_positions(2,:) == 204);
correct_mask = correct_mask_x & correct_mask_y;

% Fix all data with only correct searches
time_steps(~correct_mask) = [];
tree_size(~correct_mask,:) = [];
number_of_leaves(~correct_mask,:) = [];
number_of_cleared(~correct_mask,:) = [];


%--------------
% Compute statistics on expected time till decision
[N,X] = hist( time_steps, 1:20:1000 );

h_bar = bar( X, N./sum(N), 'r' );
stats6 = [mean(time_steps), std(time_steps,1), median(time_steps)];
fprintf('Mean: %3.2f, STD: %3.2f, Median: %d\n', mean(time_steps), std(time_steps,1), median(time_steps) );






% Load the data file for the hierarchical search simulation runs
datafile = 'allUniformDepth5.mat';    %
load( datafile );

% Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
%  tree_size
%  number_of_leaves
%  number_of_cleared
correct_mask_x = (final_positions(1,:) == 200);
correct_mask_y = (final_positions(2,:) == 200);
correct_mask = correct_mask_x & correct_mask_y;

% Fix all data with only correct searches
time_steps(~correct_mask) = [];
tree_size(~correct_mask,:) = [];
number_of_leaves(~correct_mask,:) = [];
number_of_cleared(~correct_mask,:) = [];


%--------------
% Compute statistics on expected time till decision
[N,X] = hist( time_steps, 1:20:1000 );

h_bar = bar( X, N./sum(N), 'g' );
stats5 = [mean(time_steps), std(time_steps,1), median(time_steps)];
fprintf('Mean: %3.2f, STD: %3.2f, Median: %d\n', mean(time_steps), std(time_steps,1), median(time_steps) );





% Legend
h_legend = legend(sprintf('Depth 7, Mean=%3.2f', stats7(1) ), ...
    sprintf('Depth 6, Mean=%3.2f', stats6(1) ),...
    sprintf('Depth 5, Mean=%3.2f', stats5(1) ) );










%
% Alternative figure



% Load the data file for the hierarchical search simulation runs
datafileList = {
    'allUniformDepth5', ...
    'allUniformDepth6', ...
    'allUniformDepth7',...
    };
legendEntries = {'Depth = 5', 'Depth = 6', 'Depth = 7'};


mean_vec = zeros(size(datafileList));
std_vec = zeros(size(datafileList));
median_vec = zeros(size(datafileList));
resolution_vec = [200, 204, 202];

histData = [];

for which_file = 1:length(datafileList)
    % Load data file
    load( datafileList{which_file} );
    
    
    % Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
    %  tree_size
    %  number_of_leaves
    %  number_of_cleared
    correct_mask_x = (final_positions(1,:) == resolution_vec(which_file));
    correct_mask_y = (final_positions(2,:) == resolution_vec(which_file));
    correct_mask = correct_mask_x & correct_mask_y;
    
    % Fix all data with only correct searches
    time_steps(~correct_mask) = [];
    tree_size(~correct_mask,:) = [];
    number_of_leaves(~correct_mask,:) = [];
    number_of_cleared(~correct_mask,:) = [];
    
    
    %--------------
    % Compute statistics on expected time till decision
    [N,X] = hist( time_steps, 0:100:1000 );
    
    
    histData = [histData, N'];
    
    
    % Store mean, std, median stats for this data set
    mean_vec(which_file) = mean(time_steps);
    std_vec(which_file) = std(time_steps,1);
    median_vec(which_file) = median(time_steps);
    
end         % end for(which_file)



% Compute the distributions
probData = zeros(size(histData));
for which_file = 1:length(datafileList)
    probData(:, which_file) = histData(:, which_file) ./ sum(histData(:, which_file));
    
end;

%==============
% Plot the time till decision distribution
figure;
set(gcf, 'color','white', 'Position', [360 389 667 289]);

h_bar = bar( X, probData, 1, 'grouped' );
hold on;
set(gca, 'XLim', [0, 1100] );%, 'YLim', [0 1.10*max(N./sum(N))]);
set(gca, 'FontSize', 14, 'LineWidth', 3)


xlabel('Time step, t  '); ylabel('Pr[TTD = t] ');

% Legend
h_legend = legend(sprintf('Depth 4, Mean=%3.2f', mean_vec(1) ), ...
    sprintf('Depth 5, Mean=%3.2f', mean_vec(2) ),...
    sprintf('Depth 6, Mean=%3.2f', mean_vec(3) ), 'Location', 'northwest' );

