%
%  This script plots the distribution of decision times for the
%  hierarchical approach for different initial number of nodes
%   - HierarchicalSearch_MaxDepth4 ~ 85 nodes
%   - HierarchicalSearch_MaxDepth5 ~ 341 nodes
%   - HierarchicalSearch_MaxNodes300 ~ 297 nodes
%   - HierarchicalSearch_MaxNodes150 ~ 149 nodes
%
%  Used to generate the following figures for 2011-ICRA paper
%   - 'fig-BeliefEvol_MaxNodes300.png'
%
%  Cropping notes (screen capture)
%   - (x,y,w,h) = (983,389, 610,280)
%
close all; clc; clc;

% Load the data file for the hierarchical search simulation runs
datafileList = {'HierarchicalSearch_MaxDepth4',...
    'HierarchicalSearch_MaxDepth5', ...
    'HierarchicalSearch_MaxNodes300', ...
    'HierarchicalSearch_MaxNodes150', ...
    };
legendEntries = {'Max Initial Depth = 3', 'Max Initial Depth = 4', 'Max Initial Nodes = 300', 'Max Initial Nodes = 150'};

mean_vec = zeros(size(datafileList));
std_vec = zeros(size(datafileList)); 
median_vec = zeros(size(datafileList)); 

for which_file = 1:length(datafileList)
    
    load( datafileList{which_file} );
    
    
    
    
    % Extract relevant data from the workspace (num_replicates X MAX_SIM_TIME)
    %  tree_size
    %  number_of_leaves
    %  number_of_cleared
    correct_mask_x = (final_positions(1,:) == 200.5);
    correct_mask_y = (final_positions(2,:) == 200.5);
    correct_mask = correct_mask_x & correct_mask_y;
    
    % Fix all data with only correct searches
    time_steps(~correct_mask) = [];
    tree_size(~correct_mask,:) = [];
    number_of_leaves(~correct_mask,:) = [];
    number_of_cleared(~correct_mask,:) = [];
    
    
    %--------------
    % Compute statistics on expected time till decision
    [N,X] = hist( time_steps, 1:20:1000 );
    
    %==============
    % Plot the time till decision distribution
    figure;
    set(gcf, 'color','white', 'Position', [360 389 667 289]);
    
    h_bar = bar( X, N./sum(N), 0.8 );
    hold on;
    set(gca, 'XLim', [1, size(tree_size,2)], 'YLim', [0 0.14 ]);
    set(gca, 'FontSize', 14, 'LineWidth', 3)
    
    
    xlabel('Time step, t  '); ylabel('Pr[TTD = t] ');
    ETTD(which_file) = mean(time_steps);
    fprintf('Mean: %3.2f, STD: %3.2f, Median: %d\n', mean(time_steps), std(time_steps,1), median(time_steps) );
    
    % % Legend
    % h_legend = legend(sprintf('%s, Mean=%3.2f', legendEntries{which_file}, ETTD(which_file) ) );
    
    
    
    
    
    
    % - plot mean, std, median stats
    h_text(1) = text( 500, 0.085, legendEntries{which_file} );
    h_text(2) = text( 600, 0.075, sprintf('Mean: %3.2f', mean(time_steps) ) );
    h_text(3) = text( 600, 0.065, sprintf('STD: %3.2f', std(time_steps,1) ) );
    h_text(4) = text( 600, 0.055, sprintf('Median: %d', median(time_steps) ) );
    set(h_text, 'FontSize', 14);
    set(h_text(1), 'FontWeight', 'bold');
    
    
    
    % Store mean, std, median stats for this data set
    mean_vec(which_file) = mean(time_steps);
    std_vec(which_file) = std(time_steps,1);
    median_vec(which_file) = median(time_steps);
    
    
end

% Plot search performance against number of nodes
figure;
set(gcf, 'color', 'white')

num_nodes = [
    85,             % MaxDepth=4
    341,            % MaxDepth=5
    297,            % MaxNodes=300
    149];           % MaxNodes=150

h_bar = bar(num_nodes,mean_vec);


set(gca, 'FontSize', 14);

xlabel('Number of initial nodes');
ylabel('Expected Time till Decision');
