function [adjList] = FindAdjacentCells( c_curr )

global tree;

debugFlag = false;


if (~debugFlag)
    
    % Clear adjacency list
    adjList = c_curr;
    
    
    % Add all of this cell's parents
    nodeParents = ListAllParents( c_curr );
    adjList = [adjList; nodeParents];
    
    % Add all of this cell's children
    nodeChildren = ListAllChildren(  c_curr ) ;
    adjList = [adjList; nodeChildren];
    
    
    
    
    % Need to determine all North neighbors
    nodesToN = FindAllNorthNeighbors( c_curr );
    adjList = [adjList; nodesToN];
    
    
    % Need to determine all South neighbors
    nodesToS = FindAllSouthNeighbors( c_curr );
    adjList = [adjList; nodesToS];
    
    
    % Need to determine all East neighbors
    nodesToE = FindAllEastNeighbors( c_curr );
    adjList = [adjList; nodesToE];
    
    
    % Need to determine all West neighbors
    nodesToW = FindAllWestNeighbors( c_curr );
    adjList = [adjList; nodesToW];
    
    
    % Clean up the list
    %  - code could probably be sped up
    adjList = unique(adjList);
    
    
else
    % DEBUG HACK
    disp('FindAdjacentCells: HACK');
    
    %  Assume all nodes are adjacent
    adjList = 1:size(tree,2);
    
end

end         % end function FindAdjacentCells()


%==========================================================================
function nodeParents = ListAllParents ( node )

global tree;

nodeParents = node;

% - check ancestor lineage as well
if ( tree(node).depth > 1 )
    % Add all parents as adjacent nodes as well
    done = false;
    while (~done)
        nodeParents = [nodeParents; tree(nodeParents(end)).parent];
        if ( tree(nodeParents(end)).depth == 1 )
            done = true;
        end
    end
end

end         % end function ListAllParents()


%==========================================================================
function nodeChildren = ListAllChildren ( node )

global tree;

nodeChildren = node;

if ( ~tree(node).leaf )
    
    % Since not a leaf, add children to list
    nodeChildren = [nodeChildren; tree(node).NW; tree(node).NE; tree(node).SE; tree(node).SW ];
    
    % Recurse
    nodeChildren = [nodeChildren; ListAllChildren( tree(node).NW ) ];
    nodeChildren = [nodeChildren; ListAllChildren( tree(node).NE ) ];
    nodeChildren = [nodeChildren; ListAllChildren( tree(node).SE ) ];
    nodeChildren = [nodeChildren; ListAllChildren( tree(node).SW ) ];
    
end
%
% global tree;
%
% nodeChildren = node;
%
% % - check ancestor lineage as well
% if ( tree(node).depth > 1 )
%     % Add all parents as adjacent nodes as well
%     done = false;
%     while (~done)
%         nodeParents = [nodeParents; tree(nodeParents(end)).parent];
%         if ( tree(nodeParents(end)).depth == 1 )
%             done = true;
%         end
%     end
% end

end         % end function ListAllChildren()


%==========================================================================
%---------------------
%
function neighbor = FindAllNorthNeighbors( c_curr )

% Need to determine North neighbor at greatest depth (but no greater than c_curr)
neighbor = FindNorthNeighbor( c_curr );

if ( ~isempty(neighbor) )
    neighborParents = ListAllParents( neighbor );
    neighborChildren = FindSoutherlyChildren( neighbor );
    
    neighbor = unique([c_curr; neighbor; neighborParents(2:end); neighborChildren(2:end)]);
    
    
end
end         % end function FindAllNorthNeighbors()


%---------------------
%
function neighbor = FindNorthNeighbor( c_curr )

% Finds the indices of the nodes which are adjacent to the NORTH
%  - adapted from Overmars algorithm, Ch 14, p313
%  c_curr    : current index of node
%  neighbor : list of indices for neighbors

global tree;

% Initialize list
neighbor = [];


if ( c_curr == 1 )      % check for root note
    return;
end

if ( c_curr == tree(tree(c_curr).parent).SW ) % check if SW node
    neighbor = tree(tree(c_curr).parent).NW;
    return;
end

if ( c_curr == tree(tree(c_curr).parent).SE ) % check if SE node
    neighbor = tree(tree(c_curr).parent).NE;
    return;
end

% The current node must be NW or NE node of parent
%  - so need to investigate north neighbor of parent
mu = FindNorthNeighbor( tree(c_curr).parent );

if ( isempty(mu) || tree(mu).leaf )
    neighbor = mu;
    return;
else
    
    if (c_curr == tree(tree(c_curr).parent).NW )
        neighbor = tree(mu).SW;
    else
        neighbor = tree(mu).SE;
    end
    
end

end         % end function FindNorthNeighbor()


%---------------------
%
function neighborChildren = FindSoutherlyChildren( neighbor )

global tree;

neighborChildren = neighbor;

if ( ~tree(neighbor).leaf )
    
    % Since not a leaf, add southerly children to list
    neighborChildren = [neighborChildren; tree(neighbor).SW; tree(neighbor).SE ];
    
    % Recurse
    neighborChildren = [neighborChildren; FindSoutherlyChildren( tree(neighbor).SW ) ];
    neighborChildren = [neighborChildren; FindSoutherlyChildren( tree(neighbor).SE ) ];
    
end


end         % end function FindSoutherlyChildren()


%==========================================================================
%---------------------
%
function neighbor = FindAllSouthNeighbors( c_curr )

% Need to determine South neighbor at greatest depth (but no greater than c_curr)
neighbor = FindSouthNeighbor( c_curr );

if ( ~isempty(neighbor) )
    neighborParents = ListAllParents( neighbor );
    neighborChildren = FindNortherlyChildren( neighbor );
    
    neighbor = unique([c_curr; neighbor; neighborParents(2:end); neighborChildren(2:end)]);
    
    
end
end         % end function FindAllSouthNeighbors()


%---------------------
%
function neighbor = FindSouthNeighbor( c_curr )

% Finds the indices of the nodes which are adjacent to the SOUTH
%  - adapted from Overmars algorithm, Ch 14, p313
%  c_curr    : current index of node
%  neighbor : list of indices for neighbors

global tree;

% Initialize list
neighbor = [];


if ( c_curr == 1 )      % check for root note
    return;
end

if ( c_curr == tree(tree(c_curr).parent).NW ) % check if NW node
    neighbor = tree(tree(c_curr).parent).SW;
    return;
end

if ( c_curr == tree(tree(c_curr).parent).NE ) % check if NE node
    neighbor = tree(tree(c_curr).parent).SE;
    return;
end

% The current node must be SW or SE node of parent
%  - so need to investigate south neighbor of parent
mu = FindSouthNeighbor( tree(c_curr).parent );

if ( isempty(mu) || tree(mu).leaf )
    neighbor = mu;
    return;
else
    
    if (c_curr == tree(tree(c_curr).parent).SW )
        neighbor = tree(mu).NW;
    else
        neighbor = tree(mu).NE;
    end
    
end

end         % end function FindSouthNeighbor()


%---------------------
%
function neighborChildren = FindNortherlyChildren( neighbor )

global tree;

neighborChildren = neighbor;

if ( ~tree(neighbor).leaf )
    
    % Since not a leaf, add southerly children to list
    neighborChildren = [neighborChildren; tree(neighbor).NW; tree(neighbor).NE ];
    
    % Recurse
    neighborChildren = [neighborChildren; FindNortherlyChildren( tree(neighbor).NW ) ];
    neighborChildren = [neighborChildren; FindNortherlyChildren( tree(neighbor).NE ) ];
    
end


end         % end function FindNortherlyChildren()



%==========================================================================
%---------------------
%
function neighbor = FindAllEastNeighbors( c_curr )

% Need to determine East neighbor at greatest depth (but no greater than c_curr)
neighbor = FindEastNeighbor( c_curr );

if ( ~isempty(neighbor) )
    neighborParents = ListAllParents( neighbor );
    neighborChildren = FindWesterlyChildren( neighbor );
    
    neighbor = unique([c_curr; neighbor; neighborParents(2:end); neighborChildren(2:end)]);
    
    
end
end         % end function FindAllEastNeighbors()



%---------------------
%
function neighbor = FindEastNeighbor( c_curr )

% Finds the indices of the nodes which are adjacent to the EAST
%  - adapted from Overmars algorithm, Ch 14, p313
%  c_curr    : current index of node
%  neighbor : list of indices for neighbors

global tree;

% Initialize list
neighbor = [];


if ( c_curr == 1 )      % check for root note
    return;
end

if ( c_curr == tree(tree(c_curr).parent).NW ) % check if NW node
    neighbor = tree(tree(c_curr).parent).NE;
    return;
end

if ( c_curr == tree(tree(c_curr).parent).SW ) % check if SW node
    neighbor = tree(tree(c_curr).parent).SE;
    return;
end

% The current node must be NE or SE node of parent
%  - so need to investigate east neighbor of parent
mu = FindEastNeighbor( tree(c_curr).parent );

if ( isempty(mu) || tree(mu).leaf )
    neighbor = mu;
    return;
else
    
    if (c_curr == tree(tree(c_curr).parent).NE )
        neighbor = tree(mu).NW;
    else
        neighbor = tree(mu).SW;
    end
    
end

end         % end function FindEastNeighbor()


%---------------------
%
function neighborChildren = FindWesterlyChildren( neighbor )

global tree;

neighborChildren = neighbor;

if ( ~tree(neighbor).leaf )
    
    % Since not a leaf, add southerly children to list
    neighborChildren = [neighborChildren; tree(neighbor).NW; tree(neighbor).SW ];
    
    % Recurse
    neighborChildren = [neighborChildren; FindWesterlyChildren( tree(neighbor).NW ) ];
    neighborChildren = [neighborChildren; FindWesterlyChildren( tree(neighbor).SW ) ];
    
end


end         % end function FindWesterlyChildren()



%==========================================================================
%---------------------
%
function neighbor = FindAllWestNeighbors( c_curr )

% Need to determine West neighbor at greatest depth (but no greater than c_curr)
neighbor = FindWestNeighbor( c_curr );

if ( ~isempty(neighbor) )
    neighborParents = ListAllParents( neighbor );
    neighborChildren = FindEasterlyChildren( neighbor );
    
    neighbor = unique([c_curr; neighbor; neighborParents; neighborChildren]);
    
    
end
end         % end function FindAllWestNeighbors()


%---------------------
%
function neighbor = FindWestNeighbor( c_curr )

% Finds the indices of the nodes which are adjacent to the WEST
%  - adapted from Overmars algorithm, Ch 14, p313
%  c_curr    : current index of node
%  neighbor : list of indices for neighbors

global tree;

% Initialize list
neighbor = [];


if ( c_curr == 1 )      % check for root note
    return;
end

if ( c_curr == tree(tree(c_curr).parent).NE ) % check if NE node
    neighbor = tree(tree(c_curr).parent).NW;
    return;
end

if ( c_curr == tree(tree(c_curr).parent).SE ) % check if SE node
    neighbor = tree(tree(c_curr).parent).SW;
    return;
end

% The current node must be NW or SW node of parent
%  - so need to investigate north neighbor of parent
mu = FindWestNeighbor( tree(c_curr).parent );

if ( isempty(mu) || tree(mu).leaf )
    neighbor = mu;
    return;
else
    
    if (c_curr == tree(tree(c_curr).parent).NW )
        neighbor = tree(mu).NE;
    else
        neighbor = tree(mu).SE;
    end
    
end

end         % end function FindWestNeighbor()


%---------------------
%
function neighborChildren = FindEasterlyChildren( neighbor )

global tree;

neighborChildren = neighbor;

if ( ~tree(neighbor).leaf )
    
    % Since not a leaf, add easterly children to list
    neighborChildren = [neighborChildren; tree(neighbor).NE; tree(neighbor).SE ];
    
    % Recurse
    neighborChildren = [neighborChildren; FindEasterlyChildren( tree(neighbor).NE ) ];
    neighborChildren = [neighborChildren; FindEasterlyChildren( tree(neighbor).SE ) ];
    
end


end         % end function FindEasterlyChildren()

